#include <gba_video.h>
#include <gba_interrupt.h>
#include <gba_dma.h>
#include <gba_input.h>
#include <gba_sprites.h>

#include "bg_raw.h"
#include "bg_pal.h"
#include "bg_map.h"

#include "sprites.h"
#include "palette.h"

#include "map.h"

#include "player.h"

const u8 cScreenX = 16;
const u8 cScreenY = 11;

const u8 cTileWidth = 0x10;

u8 gScreenState[cScreenY][cScreenX] =
{
	{	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, },
	{	0, 0, 4, 4, 4, 4, 6, 4, 4, 4, 0, 0, 0, 0, 0, 0, },
	{	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, },
	{	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, },
	{	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, },
	{	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, },
	{	0, 0, 0,24, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, },
	{	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, },
	{	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, },
	{	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, },
	{	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, },
};

Player gPlayer;

// global data
OBJATTR OAMCopy[128];

// a simple sprite structure
typedef struct
{
   int x,y;
   OBJATTR* oam;
   int gfxID;
} Sprite;



s16 x;
s16 y;
Sprite mario;


//sprite defines
//Atribute0 stuff
#define ROTATION_FLAG 		BIT8
#define SIZE_DOUBLE			BIT9
#define SPRITE_OFF			BIT9
#define MODE_NORMAL     	0
#define MODE_TRANSPERANT	BIT(0xA)
#define MODE_WINDOWED		BIT(0xB)
#define MOSAIC				BIT(0xC)
#define COLOR_16			0
#define COLOR_256			BIT(0xD)
#define SQUARE				0x0
#define TALL				BIT(0xF)
#define WIDE				BIT(0xE)
	
//Atribute1 stuff
#define ROTDATA(n)			((n) << 9)
#define HORIZONTAL_FLIP		BIT(0xC)
#define VERTICAL_FLIP		BIT(0xD)
#define SIZE_8				0
#define SIZE_16				BIT(0xE)
#define SIZE_32				BIT(0xF)
#define SIZE_64				BIT(0xF) | BIT(0xE)

#define OAMMem  		((u32*)0x7000000)
#define OBJ_GFXMem		((u16*)0x6010000)

void InitSprites(void)
{
   //init our sprite
   mario.oam = &OAMCopy[0];
   mario.x = 10;
   mario.y = 10;
   mario.gfxID = 0;


   //set our sprites OAM entry attributes
   mario.oam->attr0 = COLOR_256 | TALL | mario.y;  
   mario.oam->attr1 = SIZE_32 | mario.x;
   mario.oam->attr2 = mario.gfxID;

}

//turn all the sprites off
void InitOAM(void)
{
   int i;
   for(i = 0; i < 128; i++)
      OAMCopy[i].attr0 = OBJ_DISABLE;
}

// Copy our OAMcopy to OAM
void UpdateOAM(void)
{
  u16 i;

   for(i = 0; i < 128 * sizeof(OBJATTR) / 4 ; i++)
      ((u32*)OAMMem)[i] = ((u32*)OAMCopy)[i];

}

// Update the sprites OAM entry to reflect new position
void MoveSprite(OBJATTR* sp, int x, int y )
{
	sp->attr1 = sp->attr1 & 0xFE00;
	sp->attr1 = sp->attr1 | x;

	sp->attr0 = sp->attr0 & 0xFF00;
	sp->attr0 = sp->attr0 | y;
}  


void MoveMap(u16 xoffset, u16 yoffset)
{
	u8 x, y;

	BG0HOFS = xoffset & 0x0f;	// xoffset % 16
	xoffset = xoffset >> 4;		// xoffset / 16

	BG0VOFS = yoffset & 0x0f;
	yoffset = yoffset / 16;

	for(x = 0; x < cScreenX; x++)
		for(y = 0; y < cScreenY; y++)
			gScreenState[y][x] = gMap[(y + yoffset) * cMapX + x + xoffset];
}

void DrawScreenState()
{
	u8 x, y;

	u16 *tBuf = (u16 *) MAP_BASE_ADR(31);

	for(x = 0; x < cScreenX; x++)
	{
		for(y = 0; y < cScreenY; y++)
		{
			tBuf[2 * y * 32 + x * 2] = gScreenState[y][x];
			tBuf[2 * y * 32 + x * 2 + 1] = gScreenState[y][x] + 1;
			tBuf[(2 * y + 1) * 32 + x * 2] = gScreenState[y][x] + cTileWidth;
			tBuf[(2 * y + 1) * 32 + x * 2 + 1] = gScreenState[y][x] + cTileWidth + 1;
		}
	}
}

int main(void)
{
	// Set up the interrupt handlers
	InitInterrupt();
	
	// Enable Vblank Interrupt to allow VblankIntrWait
	EnableInterrupt(Int_Vblank);

	VBlankIntrWait();

	// Copy palette to vram
	DMA3COPY(bg_pal,BG_COLORS,DMA16|((int)bg_pal_size>>1));
	// Copy map to vram
	DMA3COPY(bg_raw,CHAR_BASE_ADR(0),DMA16|((int)bg_raw_size>>1));


	DMA3COPY(gMap,0x2000000,DMA16|((int)1024>>1));

	// Black backdrop
	BG_COLORS[0]=0;

	// Set bg 0 up
	BGCTRL[0]=BG_256_COLOR|SCREEN_BASE(31);

	// screen mode & background to display
	REG_DISPCNT = MODE_0 | BG0_ON | OBJ_ON | OBJ_1D_MAP;

	InitOAM();
	InitSprites();
	
	//copy in the palette
	u16 i;
	u16 *OBJPaletteMem = (u16 *) OBJ_COLORS;
	for(i = 0; i < 256; i++)
		OBJPaletteMem[i] = charPalette[i];

	//copy the sprite grahics in obj graphics mem
	for(i = 0; i < 512; i++)
		OBJ_GFXMem[i] = mario1Data[i];


	// Wait forever
	while (1)
	{


		if(!(REG_KEYINPUT & KEY_RIGHT))
			gPlayer.Right();
		if(!(REG_KEYINPUT & KEY_LEFT))
			gPlayer.Left();

		if(!(REG_KEYINPUT & KEY_UP))
			gPlayer.Up();
		if(!(REG_KEYINPUT & KEY_DOWN))
			gPlayer.Down();

		if(!(REG_KEYINPUT & KEY_A))
			gPlayer.Jump();

		gPlayer.Think();

		s16 screenx = gPlayer._x - 100;
		s16 screeny = gPlayer._y - 100;

		if(screenx < 0)
			screenx = 0;
		if(screenx > cMapPX - 240)
			screenx = cMapPX - 240;
		if(screeny < 0)
			screeny = 0;
		if(screeny > cMapPY - 160)
			screeny = cMapPY - 160;

		MoveMap(screenx, screeny);
		DrawScreenState();

		MoveSprite(mario.oam, gPlayer._x - screenx, gPlayer._y - screeny - 32);
		

		VBlankIntrWait();
		UpdateOAM();
	}
}



