
#include <gba_video.h>

#include "player.h"

const float cMoveAccel = 0.2f;
const float cMoveJumpAccel = 4.0f;
const float cMoveDamp = 0.8f;
const float cMoveMaxSpeed = 1.5f;
const float cMoveMaxFlySpeed = 3.0f;

Player::Player()
:	_x(32),
	_y(400),
	_px(32.0),
	_py(440.0),
	_vx(0.0),
	_vy(0.0),
	_groundcontact(false)
{
}

void Player::Left()
{
	_vx -= cMoveAccel;

}

void Player::Right()
{
	_vx += cMoveAccel;
}

void Player::Up()
{
	_vy -= cMoveAccel;
}

void Player::Down()
{
	_vy += cMoveAccel;
}

void Player::Jump()
{
	if(!_groundcontact)
		return;

	_groundcontact = false;

	_vy -= cMoveJumpAccel;
}

void Player::Think()
{
	if(_groundcontact)
	{
		_vx *= cMoveDamp;
		_vy = 0.0f;
	}
	else
	{
		_vy += cMoveAccel; // gravity
	}

	if(_vx > cMoveMaxSpeed)
		_vx = cMoveMaxSpeed;
	if(_vx < -cMoveMaxSpeed)
		_vx = -cMoveMaxSpeed;
	if(_vy > cMoveMaxFlySpeed)
		_vy = cMoveMaxFlySpeed;
	if(_vy < -cMoveMaxFlySpeed)
		_vy = -cMoveMaxFlySpeed;


	float px = _px + _vx;
	float py = _py + _vy;

	s16 x = (u16) px;
	s16 y = (u16) py;

	s16 mx = (x+8) >> 4;
	s16 mxr = (x+14) >> 4;
	s16 mxl = (x) >> 4;
	s16 my = (y) >> 4;
	
	if(x < 0)
	{
		_px = 0.0f;
		_vx = 0.0f;
		goto ThinkCleanup;
	}

	if(x > cMapPX - 16)
	{
		_px = cMapPX - 16.0f;
		_vx = 0.0f;
		goto ThinkCleanup;
	}
	else if(mx < cMapX-1 && gMap[(my-1) * cMapX + mxr] & 0x0f)
	{
		px = _px;
		x = (u16) px;
		_vx = 0.0f;
		mxr = (x+14) >> 4;
		mxl = (x) >> 4;
	}
	else if(mx < cMapX-1 && gMap[(my-1) * cMapX + mxl] & 0x0f)
	{
		px = _px;
		x = (u16) px;
		_vx = 0.0f;
		mxr = (x+14) >> 4;
		mxl = (x) >> 4;
	}

	if(y < 32)
	{
		_py = 32.0f;
		_vy = 0.0f;
		goto ThinkCleanup;
	}

	if(y > cMapPY)
	{
		_py = cMapPY;
		_vy = 0.0f;
		_groundcontact = true;
		goto ThinkCleanup;
	}
	else if(my < cMapY && gMap[my * cMapX + mxr] & 0x0f && _vy > 0.0f)
	{
		_vy = 0.0f;
		_px = px;
		_py = (float) (my << 4);
		_groundcontact = true;
		goto ThinkCleanup;
	}
	else if(my < cMapY && gMap[my * cMapX + mxl] & 0x0f && _vy > 0.0f)
	{
		_vy = 0.0f;
		_px = px;
		_py = (float) (my << 4);
		_groundcontact = true;
		goto ThinkCleanup;
	}
	else
	{
		_groundcontact = false;
	}

	_px = px;
	_py = py;

	_x = x;
	_y = y;

	return;

ThinkCleanup:

	_x = (u16) _px;
	_y = (u16) _py;
	return;
}


